﻿// <copyright file="AppBootstrapper.cs" company="Selectron Systems AG">
// Copyright (c) Selectron Systems AG. All rights reserved.
// </copyright>

namespace ObjectDiffer.Presentation.Evaluation
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.Reflection;
    using System.Windows;

    using Caliburn.Micro;

    using Ninject;

    /// <summary>
    /// The bootstrapper.
    /// </summary>
    public class AppBootstrapper : BootstrapperBase
    {
        /// <summary>
        /// The resolver
        /// </summary>
        private readonly IResolver resolver;

        /// <summary>
        /// Initializes a new instance of the <see cref="AppBootstrapper" /> class.
        /// </summary>
        [SuppressMessage("Microsoft.Reliability", "CA2000:Objekte verwerfen, bevor Bereich verloren geht", Justification = "Dispose is done via OnExit of this class")]
        public AppBootstrapper()
        {
            this.Initialize();
            this.resolver = new Resolver(new StandardKernel());
        }

        /// <summary>
        /// The build up.
        /// </summary>
        /// <param name="instance">
        /// The instance.
        /// </param>
        protected override void BuildUp(object instance)
        {
            this.resolver.Kernel.Inject(instance);
        }

        /// <summary>
        /// The get all instances.
        /// </summary>
        /// <param name="service">
        /// The service.
        /// </param>
        /// <returns>
        /// all found instances.
        /// </returns>
        protected override IEnumerable<object> GetAllInstances(Type service)
        {
            return this.resolver.Kernel.GetAll(service);
        }

        /// <summary>
        /// The get instance.
        /// </summary>
        /// <param name="service">
        /// The service.
        /// </param>
        /// <param name="key">
        /// The key.
        /// </param>
        /// <returns>
        /// The instance.
        /// </returns>
        protected override object GetInstance(Type service, string key)
        {
            if (service != null)
            {
                return this.resolver.Kernel.Get(service);
            }

            var type = Assembly.GetExecutingAssembly().GetType(key);
            return this.resolver.Kernel.Get(type);
        }

        /// <summary>
        /// The on startup.
        /// </summary>
        /// <param name="sender">
        /// The sender.
        /// </param>
        /// <param name="e">
        /// The e.
        /// </param>
        protected override void OnStartup(object sender, StartupEventArgs e)
        {
            this.DisplayRootViewFor<ShellViewModel>();
        }

        /// <summary>
        /// Override this to add custom behavior on exit.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The event args.</param>
        protected override void OnExit(object sender, EventArgs e)
        {
            base.OnExit(sender, e);
            this.resolver.Kernel.Dispose();
        }

        /// <summary>
        /// The select assemblies.
        /// </summary>
        /// <returns>
        /// The assemblies of the program.
        /// </returns>
        protected override IEnumerable<Assembly> SelectAssemblies()
        {
            return new[] { Assembly.GetExecutingAssembly() };
        }
    }
}
