﻿// <copyright file="DiffRowStyleSelector.cs" company="Selectron Systems AG">
// Copyright (c) Selectron Systems AG. All rights reserved.
// </copyright>

namespace ObjectDiffer.Presentation.Evaluation
{
    using System.Windows;
    using System.Windows.Controls;

    using ObjectDiffer.Presentation.Evaluation.Model;

    /// <summary>
    /// Defines the Row Style Selector for the Difference Navigation Tree
    /// </summary>
    public class DiffRowStyleSelector : StyleSelector
    {
        /// <summary>
        /// Gets or sets the changed row style.
        /// </summary>
        /// <value>
        /// The changed row style.
        /// </value>
        public Style ChangedRowStyle { get; set; }

        /// <summary>
        /// Gets or sets the inserted row style.
        /// </summary>
        /// <value>
        /// The inserted row style.
        /// </value>
        public Style InsertedRowStyle { get; set; }

        /// <summary>
        /// Gets or sets the deleted row style.
        /// </summary>
        /// <value>
        /// The deleted row style.
        /// </value>
        public Style DeletedRowStyle { get; set; }

        /// <summary>
        /// When overridden in a derived class, returns a <see cref="T:System.Windows.Style" /> based on custom logic.
        /// </summary>
        /// <param name="item">The content.</param>
        /// <param name="container">The element to which the style will be applied.</param>
        /// <returns>
        /// Returns an application-specific style to apply; otherwise, null.
        /// </returns>
        public override Style SelectStyle(object item, DependencyObject container)
        {
            if (item is DiffObject)
            {
                var diff = item as DiffObject;
                switch (diff.Status)
                {
                    default:
                        return null;
                    case DiffType.Changed:
                        return this.ChangedRowStyle;
                    case DiffType.Inserted:
                        return this.InsertedRowStyle;
                    case DiffType.Deleted:
                        return this.DeletedRowStyle;
                }
            }

            return null;
        }
    }
}
