﻿// <copyright file="ShellViewModel.cs" company="Selectron Systems AG">
// Copyright (c) Selectron Systems AG. All rights reserved.
// </copyright>

namespace ObjectDiffer.Presentation.Evaluation
{
    using System;
    using System.IO;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Xml;

    using Microsoft.Win32;

    using ObjectDiffer.Presentation.Evaluation.Model;

    using Telerik.Windows.Controls;
    using Telerik.Windows.Data;

    /// <summary>
    /// The ViewModel for the <see cref="ShellView"/>
    /// </summary>
    public class ShellViewModel : ViewModelBase
    {
        /// <summary>
        /// The random
        /// </summary>
        private static readonly Random Random = new Random();

        /// <summary>
        /// The synchronize lock
        /// </summary>
        private static readonly object SyncLock = new object();

        /// <summary>
        /// The file name
        /// </summary>
        private string fileName;

        /// <summary>
        /// The difference object tree
        /// </summary>
        private RadObservableCollection<DiffObject> diffObjectTree;

        /// <summary>
        /// Initializes a new instance of the <see cref="ShellViewModel"/> class.
        /// </summary>
        public ShellViewModel()
        {
            this.DiffObjectTree = this.Deserialize<RadObservableCollection<DiffObject>>("DataSource.xml");
        }

        /// <summary>
        /// Gets or sets the name of the file.
        /// </summary>
        /// <value>
        /// The name of the file.
        /// </value>
        public string FileName
        {
            get
            {
                return this.fileName;
            }

            set
            {
                if (value == this.fileName)
                {
                    return;
                }

                this.fileName = value;
                this.OnPropertyChanged(() => this.FileName);
            }
        }

        /// <summary>
        /// Gets or sets the difference object tree.
        /// </summary>
        /// <value>
        /// The difference object tree.
        /// </value>
        public RadObservableCollection<DiffObject> DiffObjectTree
        {
            get
            {
                return this.diffObjectTree;
            }

            set
            {
                this.diffObjectTree = value;
                this.OnPropertyChanged(() => this.DiffObjectTree);
            }
        }

        /// <summary>
        /// Opens the file.
        /// </summary>
        public void OpenFile()
        {
            FileDialog dialog = new OpenFileDialog();
            dialog.Title = "Select ASC file";
            dialog.Filter = "ASC and CFI file (*.asc;*.cfi)|*.asc;*.cfi|ASC file (*.asc)|*.asc|CFI file (*.cfi)|*.cfi|All files (*.*)|*.*";

            var ret = dialog.ShowDialog();
            if (ret.Value)
            {
                this.FileName = dialog.FileName;
                this.LoadFile();
            }
        }

        /// <summary>
        /// Gets the random number.
        /// </summary>
        /// <param name="min">The minimum.</param>
        /// <param name="max">The maximum.</param>
        /// <returns>Random number</returns>
        private static int GetRandomNumber(int min, int max)
        {
            lock (SyncLock)
            { // synchronize
                return Random.Next(min, max);
            }
        }

        /// <summary>
        /// Loads the file.
        /// </summary>
        private void LoadFile()
        {
            //var parser = new AscFileParser();
            //var tree = parser.Parse(this.FileName, 0);

            //this.DiffObjectTree = new RadObservableCollection<DiffObject> { this.BuildDiffObjectTree(tree) };
        }

        /*
        /// <summary>
        /// Builds the difference object tree.
        /// </summary>
        /// <param name="entity">The entity.</param>
        /// <returns>
        ///   <see cref="DiffObject" />
        /// </returns>
        private DiffObject BuildDiffObjectTree(IAscEntity entity)
        {
            if (entity != null)
            {
                var tree = new DiffObject(entity.Name, new ImaginaryEntity().Name, DiffType.Changed);

                if (entity is AscContainerEntity)
                {
                    var container = entity as AscContainerEntity;
                    if (container.Children.Any())
                    {
                        foreach (var child in container.Children)
                        {
                            var type = (DiffType)GetRandomNumber(0, 4);
                            var item = this.BuildDiffObjectTree(child);
                            item.Status = type;
                            tree.Children.Add(item);
                        }
                    }
                }

                return tree;
            }

            return null;
        }
         */

        /// <summary>
        /// Serializes the specified file path.
        /// </summary>
        /// <typeparam name="T">The type of object to serialize</typeparam>
        /// <param name="o">The object to serialize.</param>
        /// <param name="filePath">The file path.</param>
        private void Serialize<T>(T o, string filePath)
        {
            var serializer = new DataContractSerializer(typeof(T));
            using (var fs = new FileStream(filePath, FileMode.Create, FileAccess.ReadWrite))
            using (var writer = new XmlTextWriter(fs, Encoding.Default) { Formatting = Formatting.Indented })
            {
                serializer.WriteObject(writer, o);
            }
        }

        /// <summary>
        /// Deserializes the specified file path.
        /// </summary>
        /// <typeparam name="T">The type of object to deserialize</typeparam>
        /// <param name="filePath">The file path.</param>
        /// <returns>The deserialized object.</returns>
        private T Deserialize<T>(string filePath)
        {
            var serializer = new DataContractSerializer(typeof(T));
            using (var fs = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            using (var reader = new XmlTextReader(fs))
            {
                return (T)serializer.ReadObject(reader);
            }
        }
    }
}
